function [FACT,T,nLogP,varexpl,Lambda,RemoveFrequencies,TauW,const] = ShiftCP(X,noc,varargin)

% Usage:
%   [FACT, cost, varexpl, Lambda, RemoveFrequencies] = ShiftCP(X,noc,opts)
%
% Model:
%    x_{i1,i2,i3...,in} = \sum_d
%    a_{i1,d}*b_{i1,d-tau_{i1,d}}*c_{i3,d}*...*q_{in,d}+e_{i1,i2,i3...,in}
%
% Input:
% X                     N-way array (missing values should be set as NaN)
% noc                   number of components
% opts.
%    FACT               Initial Factors
%    constr             N x 1 vector of constraints on each mode such that
%                       constr(i) is the ith mode constraint:
%                           0: unconstrained 1: non-negativity, 2: Orthogonality
%                       Note that orthogonality cannot be imposed on mode
%                       1 and 2 whereas non-negativity should not be
%                       imposed on mode 2. (default: opts.constr=zeros(N,1);)
%    InitRun            Number of initial Runs of 30 iterations to find best initial solution (default 0)
%    TauW               valid shifts, indicate the extend to which each component can be shifted in time points.
%                       (default: opts.TauW=ones(noc,1)*[-ceil(size(X,2)/2) ceil(size(X,2)/2)];
%                       such that all conceivable shifts are possible,
%    estWholeSamples    true: estimate whole samples using cross correlation (default)
%                       false: estimate non-integer samples using
%                       gradient descent (To get non-integer delays it is recommended to first run
%                       estimation of whole samples and using this solution run the algorithm again for non-integer delays.)
%                       (default: opts.estWholeSamples=true;)
%    maxiter            maximum number of iteration (default: opts.estWholeSamples=500)
%    conv_crit          Convergence Criterion (default relative change of nLogP of 1e-6)
%    RemoveFrequencies  cell array of length noc where the dth entry is given by a Q_d x 2 matrix of frequency intervals to remove given
%                       in Hz for the dth component (default: opts.RemoveFrequencies{d}=[]; for all d)
%   Sample_rate         sampling rate for the recorded data, must be given
%                       when RemoveFrequencies are non-empty
%   ARD                 Apply automatic relevance determination to estimate
%                       number of componets (default: opts.ARD=true)
%   Lambda              1 x noc vector of initial value of regularization strength for each
%                       component 
%                       (default: Lambda=ones(1,noc)*eps*sigma_sq, i.e. hardly any regularization initially, Lambda=zeros(1,noc) if ARD=false;)
%
% Output:
%   FACT                Factor loadings, 
%   T                   Estimated time delayes, notice in the
%                       implementation negative time delays mean later
%                       onset, i.e. the model is implemented as a shift by
%                       b_{i1,d+tau_{i1,d}} instead of b_{i1,d-tau_{i1,d}}
%   nLogP               negative joint log likelihod value
%   varexpl             pct. variation explained by the model
%   Lambda              final value of Lambda
%   RemoveFrequencies   Constraints on extracted components
%   TauW                Shift constraints of extracted components
%   const               a constant term in likelihood function contributed
%                       by removed components
%
%   The CP ALS algorithm is optimized as suggested in 
%       Tomasi, G. "Practical and computational aspects in chemometric
%       data analysis", PhD Disserrorrtation, KVL 2006
%
%   Non-negativity on other mode than mode 2 implemented using the active
%   set procedure as described in:
%       Bro R., de Jong S., Journal of Chemometrics, 1997, 11, 393-401
%
%   When using this code, please cite:
%   [1] Mrup, Morten; Hansen, Lars Kai; Arnfred,Sidse M.; Lim,Lek-Heng;
%       Madsen, Kristoffer Madsen "Shift Invariant Multi-linear Decomposition of Neuroimaging Data", NeuroImage 2008
%   [2] Mrup, Morten; Hansen, Lars Kai, Kristoffer Madsen, Stefan Haufe
%       "Frequency Constrained Shifted CP Modeling of Neuroimaging Data",
%       to appear Asilomar 2011 
%
% Copyright (C) Morten Mrup, Kristoffer Hougaard Madsen and Technical University of Denmark,
% February 2007
%
% Code last updated May 11, 2011
% Code last updated April 26, 2013 to fix bug using InitRun>0
% Code last updated Juli 25, 2013 to set initial value of nLogP(1)=inf; and
% fix bug in generation of TauWMatrix
% Code last updated Juli 29, 2013 to correct that the time delays were defined as b_{i1,d+tau_{i1,d}} instead of b_{i1,d-tau_{i1,d}}. In the current implementation
%   the time delayes in the output T is defined as b_{i1,d-tau_{i1,d}} and
%   the reconstructShiftCP.m script updated accordingly. Furthermore,
%   Lambda strictly set to zeros when no ARD.
% Code last updated November 7, 2013 to change the line: 
%   conv_crit = mgetopt(opts,'convcrit',1e-6); to conv_crit = mgetopt(opts,'conv_crit',1e-6);
% Code last updated August 10, 2016 due to bug introduced when having zero delaes (instantaneous components). The code now
%   checks for the following condition if sum(TauW(d,:),2)>0 in the function estTimeAutCor
%   

if nargin >= 3, opts  =  varargin{1}; else opts  =  struct; end

% Extracted sizes of the data
nrmodes = ndims(X);
N = size(X);
if length(N)<3
   N = [N 1];
end
miss_ind=find(isnan(X));
X(miss_ind)=0;
SST=sum(X(:).^2);

% Extract algorithm parameters
RemoveFrequencies = mgetopt(opts,'RemoveFrequencies',cell(1,noc));
Sample_rate=mgetopt(opts,'Sample_rate',NaN);
ARD = mgetopt(opts,'ARD',true);
TauW = mgetopt(opts,'TauW',ones(noc,1)*[-size(X,2)/2 size(X,2)/2]);
TauWMatrix=generateTauWMatrix(TauW,N(2));
conv_crit = mgetopt(opts,'conv_crit',1e-6);
estWholeSamples = mgetopt(opts,'estWholeSamples',true);
nyT = mgetopt(opts,'nyT',ones(1,size(X,1)));
SNR= mgetopt(opts,'SNR',0);
mu=ones(1,nrmodes);
constr = mgetopt(opts,'constr',zeros(nrmodes,1));
maxiter = mgetopt(opts,'maxiter',500);
InitRun = mgetopt(opts,'InitRun',0);

sigma_sq=SST/((1+10^(SNR/10))*(prod(N)-numel(miss_ind)));

% Initialize variables
if InitRun>0
    opts.maxiter=50;
    for k=1:InitRun
        disp(['Finding best solution nr ' num2str(k) ' of ' num2str(InitRun)]);
        opts.InitRun = 0;       
        [F_tmp{k},T_tmp{k},nLogP_tmp, varexpl,Lambda_tmp{k}, RemoveFrequencies_tmp{k},~,const_tmp(k)] = ShiftCP(X,noc,opts);        
        cost(k)=nLogP_tmp(end);
    end
    [val,ind] = min(cost);
    opts.Lambda=Lambda_tmp{ind};
    RemoveFrequencies=RemoveFrequencies_tmp{ind};
    opts.const=const_tmp(ind);
    opts.FACT=F_tmp{ind};
    opts.T=T_tmp{ind};    
    noc=size(F_tmp{ind}{1},2);    
end
if isfield(opts,'FACT')
   FACT = opts.FACT;      
   if constr(1)==1           
      PP = zeros(N(1),noc);
      ZZ = repmat(1:noc,[N(1),1]);
   end
else
   for k = 1:nrmodes
       FACT=initializeFACT(N,constr,noc);
       if constr(k)==1 && k == 1
           PP = zeros(N(1),noc);
           ZZ = repmat(1:noc,[N(1),1]);           
       end
   end   
end
T = mgetopt(opts,'T',zeros(size(FACT{1})));
Lambda=mgetopt(opts,'Lambda',ones(1,noc)*eps*sigma_sq);
if ~ARD
    Lambda=0*Lambda;
end

% Transform data to frequency domain
Ns = N(2);
f = -1i*2*pi*(0:Ns-1)/Ns;
X2 = matricizing(X,2)';
Xf = fft(X2,[],2);
clear X2;
Xf = Xf(:,1:floor(size(Xf,2)/2)+1);
Xf = unmatricizing(Xf.',2,[N(1), size(Xf,2), N(3:end)]);
Nf = size(Xf);
Sf = fft(FACT{2}',[],2);
Sf = Sf(:,1:Nf(2));
f = f(1:Nf(2));

includeFreq=true(Nf(2),noc);
if ~isempty(RemoveFrequencies)        
    Frequencies=[0:Nf(2)-1]/Sample_rate*Ns;
    for d=1:noc        
        for k=1:size(RemoveFrequencies{d},1)
            ind1=find(Frequencies<RemoveFrequencies{d}(k,1)); 
            if isempty(ind1)
                ind1=1;
            else
                ind1=ind1(end);
            end
            ind2=find(Frequencies>RemoveFrequencies{d}(k,2)); 
            if isempty(ind2)
                ind2=Nf(2);
            else
                ind2=ind2(1);
            end
            includeFreq(ind1:ind2,d)=false;
        end
    end        
    Sf(~includeFreq')=0;
end
 
% Initialize algorithm variables
t = cputime;
iter = 1;
n = setdiff(1:nrmodes,[1 2]);  % Impute missing values


% Handle missing data and evaluate LS_error
krprt = ones(1,noc);
krkrt = ones(noc);
for k = 3:nrmodes
   krprt = krprod(FACT{k},krprt);
   krkrt = krkrt.*(FACT{k}'*FACT{k});
end
Rec = reconstruct(FACT{1},Sf,krprt,T,f,N);
clear F;
X(miss_ind)=Rec(miss_ind);
X2 = matricizing(X,2)';
Xf = fft(X2,[],2);
clear X2;
Xf = Xf(:,1:floor(size(Xf,2)/2)+1);
Xf = unmatricizing(Xf.',2,[N(1), size(Xf,2), N(3:end)]);    
LS_error = sum((X(:)-Rec(:)).^2);
varexpl = (SST-LS_error)/SST;
NN=sum(N(constr~=2));


% Evaluate prior
nrmFACT=zeros(1,noc);
for t=1:nrmodes
  if constr(t)~=2
    if t==2 
       if mod(N(2),2) == 0
          Sft= [Sf conj(Sf(:,end-1:-1:2))];
       else
          Sft = [Sf conj(Sf(:,end:-1:2))];
       end
       nrmFACT=nrmFACT+sum(Sft.*conj(Sft),2)'/N(2)^2;
    else
       nrmFACT=nrmFACT+sum(FACT{t}.^2);
    end
  end    
end
const=mgetopt(opts,'const',0);
nLogP(1)=inf; %0.5*LS_error/sigma_sq+0.5*sum(Lambda.*(nrmFACT+1e-6*SST/(prod(N)-numel(miss_ind))))-0.5*NN*sum(log(Lambda))+const;
dnLogP = inf;



% Algorithm Display
disp([' '])
disp(['Shifted CP Analysis'])
if estWholeSamples
   disp(['Using only integer delayes estimated by cross-spectra'])
else
   disp(['Using non-integer delayes estimated by Newton-Rhapson'])
end
disp(['A ' num2str(noc) ' component model will be fitted']);
disp([' '])
disp(['To stop algorithm press control C'])
disp([' ']);
dheader  =  sprintf('%12s | %12s | %12s | %12s | %12s | %12s | %12s','Iteration','Expl. var.','Cost func.','d_cost/cost',' Time(s)   ',' Min step   ');
dline  =  sprintf('-------------+--------------+--------------+--------------+--------------+--------------+');
disp(dline);
disp(dheader);
disp(dline);

% Main Loop
while iter<maxiter && dnLogP>=abs(nLogP(iter))*conv_crit 
   if mod(iter,25) == 0
       disp(dline);
       disp(dheader);
       disp(dline);
   end
   nLogP_old = nLogP(iter); 
   iter = iter+1;   
   told = t;

   % Project out mode 3:end to speed up algorithm 
   krprt = ones(1,noc);
   krkrt = ones(noc);
   for k = n
       krprt = krprod(FACT{k},krprt);
       krkrt = krkrt.*(FACT{k}'*FACT{k});
   end
   if nrmodes>2
       Xfp = krprt'*matricizing(Xf, [3:nrmodes]);
       Xfp = unmatricizing(Xfp,3,[Nf(1:2), noc]);
       Xtp = krprt'*matricizing(X, [3:nrmodes]);
       Xtp = unmatricizing(Xtp,3,[N(1:2), noc]);       
       krprtt = sparse(1:noc,1:noc,[ones(1,noc)]);   
   else
       Xfp = Xf;
       Xtp = X;
       krprtt = ones(1,noc);   
   end

  % Update FACT{2}    
  [Sf,mu(2)]=updateS(Sf,Xfp,X,FACT,T,f,krprtt,krkrt,N,Nf,mu(2),0.5*LS_error,constr(2),miss_ind,SST,krprt,sigma_sq*Lambda,includeFreq);
  
   % Update T 
   P.Sf = krprod(krprt,Sf.').';
   P.f = krprod(ones(size(krprt,1),1),f.').';
   P.w = ones(1,length(f));
   if mod(N(2),2) == 0
       P.w(2:end-1) = 2;
   else
       P.w(2:end) = 2;
   end
   P.w = krprod(ones(size(krprt,1),1),P.w')';
   P.sizeX2 = size(X,2);       
   P.At = FACT{1};
   P.Xft = matricizing(Xf,1);
   for m = 1:size(Xf,1)
       if estWholeSamples == 1
           [T(m,:),FACT{1}(m,:)] = estTimeAutCor(P.Xft(m,:),P.At(m,:),Sf,krprt,P.Sf,P.f,T(m,:),Nf,N,P.w,constr(1),TauWMatrix,sigma_sq*Lambda);
       else
           P.A = P.At(m,:);
           P.Xf = P.Xft(m,:);
           P.nyT = nyT(m);
           [T(m,:),nyT(m)] = update_T(T(m,:),P,TauW);           
       end       
   end

   % Center Delays
   for d=1:noc
      if sum(TauWMatrix(d,:))==N(2)
       tmean = round(mean(T(:,d)));
       T(:,d) = T(:,d)-tmean;
       Sf(d,:) = Sf(d,:).*exp(tmean*f);      
      end
   end
      
   % Update FACT{1}
   Xt = matricizing(Xtp,1);
   Ft = zeros(prod(N(1:2)),noc);
   if isempty(n)
       Rec = zeros(N);
   else
       ind = 1:N(1):prod(N(1:2));
   end
   for m = 1:size(FACT{1},1)
      Sft = Sf.*exp(T(m,:)'*f);
      if mod(N(2),2) == 0
          Sft = [Sft conj(Sft(:,end-1:-1:2))];
      else
          Sft = [Sft conj(Sft(:,end:-1:2))];
      end
      S = ifft(Sft,[],2,'symmetric');
      krpr = krprod(krprtt,S');
      krkr = krkrt.*(S*S');      
      if constr(1)==1
          [FACT{1}(m,:), q, PP(m,:), ZZ(m,:)]  = fnnlsb(krkr+sigma_sq*diag(Lambda),krpr'*Xt(m,:)',PP(m,:), ZZ(m,:));
      else
          FACT{1}(m,:) = Xt(m,:)*krpr/(krkr+sigma_sq*diag(Lambda));
      end
      if isempty(n) % reconstruct at this point if only two modalities
          Rec(m,:) = FACT{1}(m,:)*S;
      else
          Ft(ind+m-1,:) = krprod(S',FACT{1}(m,:));
      end
   end
    
   % Update FACT{3:end}
   if ~isempty(n)
       krkrt = Ft'*Ft;   
   end
   for p = n
      krpr = Ft;
      krkr = krkrt;
      k = setdiff(n,p);
      for tt = k
         krpr = krprod(FACT{tt},krpr);
         krkr = krkr.*(FACT{tt}'*FACT{tt});
      end
      Xt = matricizing(X,p);
      Xtkrpr=Xt*krpr;
      if constr(p)==1
          [FACT{p},mu(p)]=nnpg(Xtkrpr,krkr+sigma_sq*diag(Lambda),FACT{p},mu(p),20);          
      elseif constr(p)==2
          [U,Sigm,V]=svd(Xtkrpr,'econ');
          FACT{p}=U*V';
      else
          FACT{p} = Xtkrpr/(krkr+sigma_sq*diag(Lambda));
      end      
   end   
  
   % Update Lambda   
   if ARD
        nrmFACT=zeros(1,noc);
        for t=1:nrmodes
          if constr(t)~=2
            if t==2 
               if mod(N(2),2) == 0
                  Sft= [Sf conj(Sf(:,end-1:-1:2))];
               else
                  Sft = [Sf conj(Sf(:,end:-1:2))];
               end                      
               nrmFACT=nrmFACT+real(sum(Sft.*conj(Sft),2)'/N(2));
            else
               nrmFACT=nrmFACT+sum(FACT{t}.^2);
            end
          end
        end      
        if mod(iter,5)==0 % Tune Lambda every 5th iteration
            Lambda=NN./(nrmFACT+1e-6*SST/(prod(N)-numel(miss_ind)));
        end        
   end   
   
   % Impute missing values
   if ~isempty(miss_ind)
       if ~isempty(n)
          Rec = FACT{p}*krpr';
          Rec = unmatricizing(Rec,p,N);
       end
       X(miss_ind)=Rec(miss_ind);
       X2 = matricizing(X,2)';
       Xf = fft(X2,[],2);
       clear X2;
       Xf = Xf(:,1:floor(size(Xf,2)/2)+1);
       Xf = unmatricizing(Xf.',2,[N(1), size(Xf,2), N(3:end)]);
       Rec(miss_ind)=0;
       LS_error = SST-2*sum(X(:).*Rec(:))+sum(Rec(:).^2);
   else
       if isempty(n)
           LS_error=sum(sum((X-Rec).^2));
       else
           LS_error = SST-2*sum(sum(Xtkrpr.*FACT{p}))+sum(sum(krkr.*(FACT{p}'*FACT{p})));
       end       
   end
   
   % Remove components pruned to zero
   ind=find(nrmFACT==0);
   if ~isempty(ind)
       disp(['removing ' num2str(length(ind)) ' component(s)']);
       for k=1:nrmodes
           FACT{k}(:,ind)=[];
       end
       Sf(ind,:)=[];
       T(:,ind)=[];
       Lambda(ind)=[];
       TauW(ind,:)=[];
       TauWMatrix(ind,:)=[];
       nrmFACT(ind)=[];       
       includeFreq(:,ind)=[];
       RemoveFrequencies(ind)=[]; 
       if constr(1)==1
          PP(:,ind)=[];
          ZZ(:,ind)=[];
       end
       const=const-0.5*length(ind)*NN*log(NN/1e-9);
       noc=noc-length(ind);
   end
   
   if ARD
       nLogP(iter)=0.5*LS_error/sigma_sq+0.5*sum(Lambda.*(nrmFACT+1e-6*SST/(prod(N)-numel(miss_ind))))-0.5*NN*sum(log(Lambda))+const;
   else
       nLogP(iter)=0.5*LS_error/sigma_sq;
   end
   
   % Display iteration
   dnLogP = nLogP_old-nLogP(iter);
   t = cputime;
   tim = t-told;
   varexpl = (SST-LS_error)/SST;
   disp(sprintf('%12.0f | %12.4f | %12.4e | %12.4e | %12.4f | %12.4e ',iter, varexpl,nLogP(iter),dnLogP/abs(nLogP(iter)),tim,min(mu(constr==1))));   
end

% Set output variables
if mod(N(2),2) == 0
    Sf = [Sf conj(Sf(:,end-1:-1:2))];
else
    Sf = [Sf conj(Sf(:,end:-1:2))];
end
FACT{2} = ifft(Sf,[],2,'symmetric')';


% ------------------------------------------------------------------------
% Parser for optional arguments
function var  =  mgetopt(opts, varname, default, varargin)
if isfield(opts, varname)
   var  =  getfield(opts, varname);
else
   var  =  default;
end
for narg  =  1:2:nargin-4
   cmd  =  varargin{narg};
   arg  =  varargin{narg+1};
   switch cmd
       case 'instrset',
           if ~any(strcmp(arg, var))
               fprintf(['Wrong argument %s  =  ''%s'' - ', ...
                   'Using default : %s  =  ''%s''\n'], ...
                   varname, var, varname, default);
               var  =  default;
           end
       otherwise,
           error('Wrong option: %s.', cmd);
   end
end

%-----------------------------------------------------
function [Sf,muS]=updateS(Sf,Xfp,X,FACT,T,f,krprtt,krkrt,N,Nf,muS,cost,constr,miss_ind,SST,krprt,Lambda,includeFreq)  
  
  if constr==2
      disp('Orthogonality not allowed on mode 2 - ignoring constraint');
  end
  Sfg=zeros(size(Sf));
  for m = 1:Nf(2)
      comp=find(includeFreq(m,:));
      Xft = Xfp(:,m,:);
      F = FACT{1}(:,comp).*exp(T(:,comp)*f(m));
      krpr = krprod(krprtt(:,comp),F);
      krkr = krkrt(comp,comp).*(F'*F);
      if constr==1
          Sfg(comp,m) = krkr*Sf(comp,m)-krpr'*Xft(:);          
      else
          Sf(comp,m) = (krkr+diag(Lambda(comp)))\krpr'*Xft(:);
      end
  end
  if constr==1 % Use projected gradient for non-negativity constraint
      if mod(N(2),2) == 0
          Sfg = [Sfg conj(Sfg(:,end-1:-1:2))];          
      else
          Sfg = [Sfg conj(Sfg(:,end:-1:2))];          
      end
      if mod(N(2),2) == 0
          S = [Sf conj(Sf(:,end-1:-1:2))];
      else
          S = [Sf conj(Sf(:,end:-1:2))];
      end            
      S = ifft(S,[],2,'symmetric');
      S(S<0) = 0;      
      Sg = (ifft(Sfg,[],2,'symmetric')/N(2)+diag(Lambda)*S)/(trace(krkr)/size(krkr,1));              
      stop = 0;
      cost=cost+0.5*sum(Lambda*S.^2);
      while ~stop
           St  =  S-muS*Sg;
           St(St<0)=0;           
           Sf  =  fft(St,[],2); 
           Sf  =  Sf(:,1:Nf(2));
           Rec  =  reconstruct(FACT{1},Sf,krprt,T,f,N);
           if ~isempty(miss_ind) % do not count areas missing in objective
               Rec(miss_ind)=0;
           end
           costnew = 0.5*SST-sum(X(:).*Rec(:))+0.5*sum(Rec(:).^2)+0.5*sum(Lambda*St.^2);
           if costnew<= cost*(1+1e-9) 
                cost = costnew;
                muS = muS*1.2;
                stop = 1;
           else
                muS = muS/2;
           end
      end      
  end                  

% -------------------------------------------------------------------------
function [T,nyT,cost] = update_T(T,P,TauW)
   nyT = P.nyT;
   Sf = P.Sf;
   A = P.A;
   sizeX2 = P.sizeX2;
   Xf = P.Xf;
   f = P.f;
   w = P.w;  
   Recfd = zeros(size(A,1),size(Sf,2),size(A,2));
   for d = 1:size(A,2)
       Recfd(:,:,d) = (repmat(A(:,d),[1 length(f)]).*exp(T(:,d)*f)).*repmat(Sf(d,:),[size(A,1),1]);
   end
   Recf = sum(Recfd,3);
   Q = Recfd.*repmat(conj(Xf-Recf),[1,1,size(Recfd,3)]);
   grad = squeeze(sum(repmat((w.*f),[size(Q,1),1,size(Q,3)]).*(conj(Q)-Q),2))';  
   ind1 = find(w == 2); % Areas used twice
   ind2 = find(w == 1); % Areas used once
   cost_old = norm(Xf(:,ind1)-Recf(:,ind1),'fro')^2;
   cost_old = cost_old+0.5*norm(Xf(:,ind2)-Recf(:,ind2),'fro')^2;
   keepgoing = 1;
   Told = T;
   while keepgoing
       T = Told-nyT*grad;
       for d=1:length(T)
          if T(d)<TauW(d,1)
              T(d)=TauW(d,1);
          end
          if T(d)>TauW(d,2)
              T(d)=TauW(d,2);
          end
       end
       for d = 1:size(A,2)
           Recfd(:,:,d) = (repmat(A(:,d),[1 length(f)]).*exp(T(:,d)*f)).*repmat(Sf(d,:),[size(A,1),1]);
       end
       Recf = sum(Recfd,3);
       cost = norm(Xf(:,ind1)-Recf(:,ind1),'fro')^2;
       cost = cost+0.5*norm(Xf(:,ind2)-Recf(:,ind2),'fro')^2;
       if cost<= cost_old
           keepgoing = 0;
           nyT = nyT*1.2;
       else
           keepgoing = 1;
           nyT = nyT/2;
       end
   end
   T = mod(T,sizeX2);
   ind = find(T>floor(sizeX2/2));
   T(ind) = T(ind)-sizeX2;

% -------------------------------------------------------------------------
function [T,A] = estTimeAutCor(Xf,A,Sf,krpr,krSf,krf,T,Nf,N,w,constr,TauW,Lambda)

noc = size(A,2);
if mod(N(2),2) == 0
    sSf = 2*Nf(2)-2;
else
    sSf = 2*Nf(2)-1;
end
t1 = randperm(size(A,1));
t2 = randperm(noc);
for k = t1   
    Resf=Xf(k,:)-A(k,:)*(krSf.*exp(T(k,:)'*krf));
    for d = t2  
        if sum(TauW(d,:),2)>0
           Resfud = Resf+A(k,d)*(krSf(d,:).*exp(T(k,d)*krf));
           Xft = squeeze(unmatricizing(Resfud,1,[1 Nf(2) prod(Nf(3:end))]));
           if size(krpr,1) == 1
               Xd = Xft;
           else
               Xd = krpr(:,d)'*Xft.';
           end
           C = Xd.*conj(Sf(d,:));
           if mod(N(2),2) == 0
               C = [C conj(C(end-1:-1:2))];
           else
               C = [C conj(C(end:-1:2))];
           end
           C = ifft(C,'symmetric');
           C = C.*TauW(d,:);       
           if constr
               [y,ind] = max(C);
           else
               [y,ind] = max(abs(C));
           end
           T(k,d) = (ind-sSf)-1;
           A(k,d) = C(ind)/(sum(w.*(krSf(d,:).*conj(krSf(d,:))))/sSf+Lambda(d));
           if abs(T(k,d))>(sSf/2)
               if T(k,d)>0
                   T(k,d) = T(k,d)-sSf;
               else
                   T(k,d) = T(k,d)+sSf;
               end
           end
           Resf = Resfud-A(k,d)*(krSf(d,:).*exp(T(k,d)*krf));
        end
   end
end

% -------------------------------------------------------------------------
function Y = matricizing(X,n)
% Turn tensor to matrix along n'th mode
if isempty(n)
   Y = X;
else
   sX = size(X);
   N = ndims(X);
   n2 = setdiff(1:N,n);
   Y = reshape(permute(X, [n n2]),prod(sX(n)),prod(sX(n2)));
end

% -------------------------------------------------------------------------
function A = krprod(B,C);
% Khatri Rao product
sb = size(B,1);
sc = size(C,1);
A = zeros(sb*sc,size(B,2));
for k = 1:size(B,2)
    A(:,k) = reshape(C(:,k)*B(:,k)',sb*sc,1);
end

% -------------------------------------------------------------------------
function X = unmatricizing(X,n,D)
% Inverse function of matricizing
ind = 1:length(D);
ind(n) = [];
if n == 1
    perm = [1:length(D)];
else
    perm = [2:n(1) 1 n(end)+1:length(D)];
end
  
X = permute(reshape(X,D([n 1:n(1)-1 n(end)+1:length(D)])),perm);


% -------------------------------------------------------------------------
function [x,w,P,Z]  =  fnnlsb(XtX,Xty,P_old,Z_old,tol)
%FNNLSb	Non-negative least-squares.
%
%     	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	%							 %
%	% Please note this version of NNLS is for advanced users %
%	% Please refer to the m-file FNNLS for a simpler fast    %
%	% version of NNLS				         %
%	%						         %
%     	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% 	Adapted from NNLS of Mathworks, Inc.
%
%	x  =  fnnlsb(A,b) returns the vector X that solves x  =  pinv(A)*b
%	in a least squares sense, subject to x > =  0. 
%
%	A default tolerance of TOL  =  MAX(SIZE(A)) * NORM(A,1) * EPS
%	is used for deciding when elements of x are less than zero.
%	This can be overridden with x  =  fnnlsb(A,b,P_old,Z_old,TOL).
%
%	[x,w]  =  fnnlsb(A,b) also returns dual vector w where
%	w(i) < 0 where x(i)  =  0 and w(i)  =  0 where x(i) > 0.
%       [X,W,P,Z]  =  fnnlsb(A,b) also returns the index set P and Z
%	that explicitly tells which constraints (variables) are
% 	active, which are not. These can be used as subsequent 
%	input to fnnlsb if only minor changes are expected in a new 
%	problem. P designates the passive (unconstrained) set, while
%	designates the active constraints. For initialisation use
% 	P = zeros(size(Xty));
%	Z = [1:length(Xty)]';
%
%	See also FNNLS and NNLS

%	L. Shure 5-8-87
%	Revised, 12-15-88,8-31-89 LS.
%	(Partly) Copyright (c) 1984-94 by The MathWorks, Inc.

%	Modified by R. Bro 5-7-96 according to
%       Bro R., de Jong S., Journal of Chemometrics, 1997, 11, 393-401
% 	Corresponds to the FNNLSb algorithm in the paper
%
%
%	Rasmus bro
%	Chemometrics Group, Food Technology
%	Dept. Dairy and Food Science
%	Royal Vet. & Agricultural
%	DK-1958 Frederiksberg C
%	Denmark
%	rb@kvl.dk
%	http://newton.foodsci.kvl.dk/rasmus.html

%  Reference:
%  Lawson and Hanson, "Solving Least Squares Problems", Prentice-Hall, 1974.

% initialize variables
if nargin < 5
    tol  =  10*eps*norm(XtX,1)*max(size(XtX));
end
[m,n]  =  size(XtX);
P  =  P_old(:)';
PP  =  find(P);
Z  =  Z_old(:)';
ZZ = find(Z);
iter  =  0;
itmax  =  30*n;
z = zeros(n,1);
x = z;
z(PP) = (Xty(PP)'/XtX(PP,PP)');

    while any((z(PP) <=  tol)) & iter < itmax
        iter  =  iter + 1;
        QQ  =  find((z <=  tol) & P');
        alpha  =  min(x(QQ)./(x(QQ) - z(QQ)));
        x  =  x + alpha*(z - x);
        ij  =  find(abs(x) < tol & P' ~=  0);
        Z(ij) = ij';
        P(ij) = zeros(1,max(size(ij)));
        PP  =  find(P);
        ZZ  =  find(Z);
        nzz  =  size(ZZ);
        z(PP) = (Xty(PP)'/XtX(PP,PP)');
        z(ZZ)  =  zeros(nzz(2),nzz(1));
        z = z(:);
    end
    x  =  z;
    w  =  Xty-XtX*x;

% set up iteration criterion
iter  =  0;

% outer loop to put variables into set to hold positive coefficients
while any(Z) & any(w(ZZ) > tol)
    [wt,t]  =  max(w(ZZ));
    t  =  ZZ(t);
    P(1,t)  =  t;
    Z(t)  =  0;
    PP  =  find(P);
    ZZ  =  find(Z);
    nzz  =  size(ZZ);
    z(PP) = (Xty(PP)'/XtX(PP,PP)');
    z(ZZ)  =  zeros(nzz(2),nzz(1))';
    z = z(:);
    while any((z(PP) <=  tol)) & iter < itmax
        iter  =  iter + 1;
        QQ  =  find((z <=  tol) & P');
        alpha  =  min(x(QQ)./(x(QQ) - z(QQ)));
        x  =  x + alpha*(z - x);
        ij  =  find(abs(x) < tol & P' ~=  0);
        Z(ij) = ij';
        P(ij) = zeros(1,max(size(ij)));
        PP  =  find(P);
        ZZ  =  find(Z);
        nzz  =  size(ZZ);
        z(PP) = (Xty(PP)'/XtX(PP,PP)');
        z(ZZ)  =  zeros(nzz(2),nzz(1));
        z = z(:);
    end
    x  =  z;
    w  =  Xty-XtX*x;
end
x = x(:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function X = reconstruct(A,Sf,Q,T,f,N)

if mod(N(2),2) == 0
    X = zeros(size(A,1),2*length(f)-2,size(Q,1));
else
    X = zeros(size(A,1),2*length(f)-1,size(Q,1));
end
for i = 1:size(A,1)
   Sft = Sf.*exp(T(i,:)'*f); 
   if mod(N(2),2) == 0
       Sft = [Sft conj(Sft(:,end-1:-1:2))];
   else
       Sft = [Sft conj(Sft(:,end:-1:2))];
   end
   St = real(ifft(Sft,[],2));
   X(i,:,:) = St'*krprod(A(i,:),Q)';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function FACT=initializeFACT(N,constr,noc)
for i=1:length(N)   
   if constr(i)==1
      FACT{i}=rand(N(i),noc); 
   elseif constr(i)==2
      [FACT{i}, s_dummy, v_dummy]=svd(randn(N(i),noc));
   else
       FACT{i}=randn(N(i),noc);
   end              
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%          
function TauWMatrix=generateTauWMatrix(TauW,N2)
    TauWMatrix=zeros(size(TauW,1),N2);
    for d=1:size(TauW,1)
       TauWMatrix(d,1:TauW(d,2))=1;
       TauWMatrix(d,end:-1:end+TauW(d,1)+1)=1;
    end
    
    
